#include "Config.h"

#ifndef INPUT_H
#define INPUT_H


// ********************************************************************************************************************************
// *******************************   Joystick  **************************************************************************************
// ********************************************************************************************************************************


class Joystick 
{
  public: 
    
    Joystick (
                uint8_t pin,                            // pin 
                uint16_t neutral,                       // position neutre entre 0 et 1023
                uint8_t deadBand,                       // zone neutre
                int8_t direction,                        // direction -1 ou 1
                uint16_t max,
                uint16_t min
                );
    
    int16_t get(); // renvoie la valeur de la position 
    
    uint16_t rawGet();
     
  private:
    
    uint8_t   _pin;
    uint16_t  _neutral, _read, _min, _max;    //neutral position (between 0 et 1023) / Analogic read value / minimum  / maximum value 
   
    uint8_t   _deadBand;
    int8_t    _direction ;
   
};




// ********************************************************************************************************************************
// *******************************   Switch  **************************************************************************************
// ********************************************************************************************************************************


enum switchType { PIN, JOY};               


class Switch 
{
  public: 
    
    static boolean enableKeys();
    static boolean anyShortPress();
    static void resetAnyShortPress();
    
    Switch(uint8_t initPin);

    Switch(Joystick *joy, int16_t th);
    
    void begin();
    void handle(); 
    boolean state();  
    boolean shortPress();
    boolean longPress();

  private:    
    uint32_t _currentMillis;
    
    switchType _type;
    uint8_t _pin;

    Joystick *_joy ;
    int16_t   _th;
    
    uint32_t  _timePress;
    boolean  _state, _lastState, _shortPress, _longPress, _resetLongPress  ;
    
    static boolean _anyShortPress;                  // pour n'importe quel bouton
    
    static uint32_t  _lastPress;                    // instant de derniers press
};



// ********************************************************************************************************************************
// *******************************   Gyro  **************************************************************************************
// ********************************************************************************************************************************



#include <Wire.h>
#include <Kalman.h>
#include <I2Cdev.h>
#include <MPU6050.h>

class Gyro : public MPU6050
{
  public : 

      void begin () ; 
      void handle() ;
      
      double getRoll();
      double getPitch();

      Vector Joystick();

      int16_t ay();
      int16_t ax();
      
  private : 
      static boolean _gyroEnable;
      
      int16_t _ax, _ay, _az, _gx, _gy, _gz, _mx, _my, _mz; // raw value from MPU

      double _gyroRoll, _gyroPitch, _gyroYaw;

      double _gxRate, _gyRate; // Angle calculate using the gyro only

      double _kalRoll, _kalPitch; // Calculated angle using a Kalman filter
      
      elapsedMillis _t;
      
      double   _roll, _pitch;

      int16_t  _pitchDelay; 
      
      Kalman kalmanX; // Create the Kalman instances
      Kalman kalmanY;

      uint32_t timer;
     
};

#endif
