
#include <Arduino.h>
#include "Config.h"
#include "Input.h"
#include <SPI.h>
#include <FastSSD1351.h>
#include "Menu.h"
#include <TimeLib.h> 
#include "TimeClock.h" 
#include "Player.h" 
#include "Game.h" 
#include "World.h"
#include "WData.h"
#include <EEPROM.h>
#include "MPU6050.h"
#include <Snooze.h>

extern FastSSD1351 oled; 
extern TimeClock clock; 

extern Joystick  joyX;                 // declaration objet joystick
extern Joystick  joyY;  

extern Switch  swUp, swMid, swDown, swJoy ;                 // declaration objet switch

extern Switch  swJoyRight, swJoyLeft, swJoyDown, swJoyUp;            // declaration objet switch sur mouvemet joystick

extern World *worlds;
extern Player player;

extern Conf conf;

extern Gyro mpu;

extern elapsedMillis sleepTime;
extern SnoozeBlock config_teensy3x;

// global variable for ITEMS ------------------------------------------------------------ 

Item* currentMenu = NULL;
boolean (*currentProg)() = NULL ;


// Building menus
   // MENU WORLD                Title,             
      Item worldsItems[] = {  Item      (&worlds[0] ),
                              Item      (&worlds[1] ),
                              Item      (&worlds[2] ),  
                              Item      (&worlds[3] ) , 
                              Item      ("Reset   ",  resetWorlds )
                            } ; // tableau d'items pour menu2
      
   // parameters MENU 
      Item parametersItems[] = {  Item           ("Sounds" ,            &conf.enableSound ),
                                  Item           ("AutoScroll"  ,       &conf.enableGyro ),
                                  Item           ("PlayGyro"  ,         &conf.playGyro ),
                                  Item           ("LoopTime"  ,         &conf.enableLoopTime ),
                                  Item           ("ShadowMode"  ,       &conf.enableShadowMode ),
                                  Item           ("SleepMode"  ,        &conf.enableDeepSleepMode ),
                                  Item           ("Debug",              debug ), 
                                  Item           ("About"  ,            progAbout ) 
                                } ; // tableau d'items pour menu1
      
   // Main MENU 
      Item mainMenuItems[] = {  
                                Item           ("Game",             game ),
                                Item           ("Worlds Menu",      nbworlds +1, worldsItems ) , 
                                Item           ("Clock",            TimeClock::display), 
                                Item           ("Options",   8,     parametersItems ), 
                                Item           ("GoToSleep",        goToSleep ), 
                                Item           ("Save",             saveEeprom )
                                } ; // tableau d'items pour mainMenu
      
      Item mainMenu  = Item ("Main Menu" , 6, mainMenuItems);    // creation de l'item du mainMenu






// managing menu 
void menuBegin()   
{
  currentMenu = &mainMenu;           // we activate the main menu
  //currentMenu = parametersItems ;  
  
  for (byte i=0; i< nbworlds; i++) worldsItems[i]._title = (worldsItems[i]._world)->getName();   // initialize menu world titles with worlds names
  for (byte i=0; i<conf.maxLevel+1; i++) worldsItems[i]._world->setEnable(HIGH);                //  initialize worlds available or not                  

  currentProg = game ;                // we start directly in the game 
  //currentProg = debug;
}

void menuManage()   
 {
 if ( currentProg != NULL) 
        {
        if ( (*currentProg)() )  //  // / execute program  (if currentProg is not NULL)  and return le exit condition 
                          currentProg = NULL ;   // exit condition  is HIGH : return to menu with currentProg == NULL 
        }
        
 else // currentProg == NULL  -> we display the menu 
   {
     uint8_t n = currentMenu->_itemNb;                            // nb of items of current menu 
        
    // selecction of items in cuurent menu  
        if (swJoyDown.shortPress()) currentMenu->_index = currentMenu->_index +1; 
                
        if (currentMenu->_index==n) currentMenu->_index = 0;
        
        if (swJoyUp.shortPress()) currentMenu->_index = currentMenu->_index -1;
                
        if (currentMenu->_index==255) currentMenu->_index=n-1;
    
    // background //oled.fillRect(0, 0, 128, 128, BLACK); 
        
        player.play(
                              worlds[conf.level],
                              { 0 , 0 } ,                      // joystick position 
                              LOW,      // jump
                              LOW        // action
                              );
                 
        oled.screenShade (2);
        
    // display title
        
        oled.printXYColorSize (currentMenu->_title,          0,        0,  WHITE,  1); 
        oled.printXYColorSize ( clock.getTimeString (), 127-8*6, 0, WHITE, 1);
        oled.fillRect(0, 8, 128, 1, WHITE); // ligne de separation 
    
    // Affichage des Items en fonction du type d'item
        
        for (uint8_t i=0; i<n; i++) 
            {  
            Item *item = &(currentMenu->_items[i]);
            
            String s = item->_title;
            
            if (i!=currentMenu->_index) oled.printXYColorSize ( s ,   1, 11+9*i,   WHITE,  1); 
            else 
                  {
                  oled.fillRect(0, 10+9*i, s.length()*6, 9, WHITE);     // rectangle blanc
                  oled.printXYColorSize (s,   1, 11+9*i,   BLACK,  1); 
                  }    

            if (item->_type == WORLD) 
                              {
                                    String e;
                                    if (item->_world->getEnable()) e =" OK";
                                    else e =" --";
                                    
                                    oled.printXYColorSize(e, 128-18, 11+9*i,  WHITE, 1);
                              }
            
            if (item->_type == BOOLEANVALUE) 
                              {
                                    String b;
                                    if (*(item->_booleanValue)) b = "ON"; else b = "OFF";
                                    oled.printXYColorSize(b, 128-18, 11+9*i,  WHITE, 1);
                              }
            }
    

     Item *currentItem = &(currentMenu->_items[currentMenu->_index]);  // pointeur sur l'item courant
     
     itemType t = currentItem->_type;                                   // type de l'item courant

    // Actions -------------------------------------------
       
       if ( t == WORLD ) 
          {
              
              
              if (  
                    currentItem->_type == WORLD       && 
                    currentItem->_world->getEnable()
                    ) 
                          {
                             conf.level = currentMenu->_index;  // definition du monde à la volée 
                             player.begin(worlds[conf.level]);
                          }
              
              if (swJoyRight.shortPress() || swDown.shortPress())  // pour aller directement dans le jeu à partir du menu worlds
                          {
                            //if (conf.enableSound) tone (5,40,10);
                            currentMenu = &mainMenu;
                            currentProg=game;
                            player.begin(worlds[conf.level]); //  *currentItem->_world
                          }
          }
       else if ( t == MENU)
               {
                 if (swJoyRight.shortPress() || swDown.shortPress()) currentMenu = currentItem;
               }
             
             else if (t == PROG) 
                      { 
                        if (swJoyRight.shortPress() || swDown.shortPress()) currentProg=currentItem->_prog;  // affectation du pointeur du menu sur programme courant
                     }

                  else if (t == BOOLEANVALUE)
                        {
                          if (swJoyRight.shortPress() || swDown.shortPress()) 
                                {
                                  boolean *val = currentItem->_booleanValue;
                                  *val = !*val;
                                }
                        }
    // back menu
     if (swJoyLeft.shortPress() || swUp.shortPress())     // back to upper menu
                          {
                          if (currentMenu==&mainMenu) currentProg=currentMenu->_items[0]._prog;
                          else currentMenu = currentMenu->_backMenu; // &mainMenu;
                          }
                          
   }
   
                                    // si condition de sortie LOW  continue à exécuter le programme                                 
}

// *********************** class Item *************************************************************************************
// Constructor MENU -------------------------------------------------------------------
    Item::Item (
               String title,             // menu title
               uint8_t itemNb,           // items nb 
               Item *items               // pointer of items array
                )
                  : _title(title),
                    _itemNb(itemNb),
                    _items(items)
     {
     _type = MENU;
     for (byte i=0; i < _itemNb ; i++) if (_items[i]._type==MENU) _items[i].setBackMenu(i, this);
     }
     
//  Constructor VALUE  -------------------------------------------------------------------
    Item::Item (
                    String title,             // menu title
                    boolean *booleanValue
                ) 
                      : _title(title),
                        _booleanValue(booleanValue)   
     {
         _type = BOOLEANVALUE;
     }

//  Constructor PROG  -------------------------------------------------------------------
    Item::Item (
                    String title,             // menu title
                    boolean (*prog)()         // pointeur de la routine executée
                ) 
                      : _title(title),
                        _prog(prog)           // Pointer of the prog to be executed 
     {
     _type = PROG;
     }


//  Constructor WORLDS  -------------------------------------------------------------------
   Item::Item (   World  *world    ) : _world(world) // pointeur du world
               {
                _type = WORLD;
               }

// Method ------------------------------------------------------------------------
  
  void Item::setBackMenu(uint8_t index, Item * menu)
        {
          _backIndex = index;
          _backMenu = menu;
        }
 

// ******************************* PROGRAMMES *************************************************************

boolean debug() 
{
  oled.fillRect(0, 0, 128, 128, BLACK); 
     
   //oled.fillRect(_pos.x-_scroll.x, _pos.y-_scroll.y, 24, 32, BLACK);  // TEST ***********************************************
//              RRRRRGGGGGGBBBBB
#define GREY  0b1000010000010000

 oled.printXYColorSize ("Gyro"  ,   0, 0,   WHITE,  1); 
  oled.printXYColorSize ("ax:"  ,   0, 1*8,   GREY,  1);  oled.printXYColorSize (String(mpu.ax())  ,   4*6, 1*8,   GREY,  1);
  //oled.print ("AY :"  ,   0, 2*8,   GREY,  1);  oled.print (String(_ay )  ,   5*6, 2*8,   GREY,  1);
  //oled.print ("AZ :"  ,   0, 3*8,   GREY,  1);  oled.print (String(_az)  ,   5*6, 3*8,   GREY,  1);

  oled.printXYColorSize ("ay :"  ,   10*6, 1*8,   GREY,  1);  oled.printXYColorSize (String(mpu.ay())  ,   14*6, 1*8,   GREY,  1);
  //oled.print ("GY :"  ,   0, 6*8,   GREY,  1);  oled.print (String(_gy )  ,   5*6, 6*8,  GREY,  1);
  //oled.print ("GZ :"  ,   0, 7*8,   GREY,  1);  oled.print (String(_gz )  ,   5*6, 7*8,   GREY,  1);
 
  oled.printXYColorSize ("roll"  ,   0, 2*8,   GREY,  1);  oled.printXYColorSize (String( mpu.getRoll() )  ,   4*6, 2*8,   GREY,  1);
  oled.printXYColorSize ("pitch"  ,   64, 2*8,   GREY,  1);  oled.printXYColorSize (String( mpu.getPitch() )  ,   64 + 5*6, 2*8,   GREY,  1);

  oled.printXYColorSize ("Time"  ,   0, 3*8,   WHITE,  1);
  uint32_t t = millis()/1000;
  String st =  String(hour(t)) + "h" + String(minute(t)) + "m" + String(second(t)) + "s" ;
  oled.printXYColorSize ("OnTime  :"  ,   0, 4*8,   GREY,  1);  oled.printXYColorSize ( st  ,   11*6, 4*8,   GREY,  1);
  
  t = sleepTime/1000; 
  st = String(minute(t)) + "m" + String(second(t)) + "s" ; 
  oled.printXYColorSize ("Sleep   :"  ,   0, 5*8,   GREY,  1);  oled.printXYColorSize ( st    ,   11*6, 5*8,   GREY,  1);

  oled.printXYColorSize ("Joystick"  ,   0, 6*8,   WHITE,  1);
  
  oled.printXYColorSize ("JoyRaw  :"  ,   0, 7*8,   GREY,  1);  oled.printXYColorSize (String( joyX.rawGet() )  ,   11*6, 7*8,   GREY,  1);   oled.printXYColorSize (String( joyY.rawGet() )  ,   16*6, 7*8,   GREY,  1);
  oled.printXYColorSize ("JoyOut  :"  ,   0, 8*8,   GREY,  1);  oled.printXYColorSize (String( joyX.get() )     ,   11*6, 8*8,   GREY,  1);   oled.printXYColorSize (String( joyY.get() )     ,   16*6, 8*8,   GREY,  1);

  oled.printXYColorSize ("Game"  ,    0, 9*8,   WHITE,  1);
  oled.printXYColorSize ("PosPix  :"  ,   0, 10*8,   GREY,  1);  oled.printXYColorSize (String( player.getPos().x),   11*6, 10*8,   GREY,  1); oled.printXYColorSize (String( player.getPos().y),   16*6, 10*8,   GREY,  1);
  oled.printXYColorSize ("PosBlock:"  ,   0, 11*8,   GREY,  1);  oled.printXYColorSize (String( player.getPos().x / 8),   11*6, 11*8,   GREY,  1); oled.printXYColorSize (String( player.getPos().y / 8),   16*6, 11*8,   GREY,  1);

  if (digitalRead(INT_PIN) ) oled.fillRect(64, 127-10, 10, 10, WHITE);
  
  /*
    // debug  *******************************************************************************
        
        Vector test = _pos;
        test.y += 32;
        oled.print (String(worlds[conf.level].blockType (test)), 127-12, 127-8, WHITE,  1);
        if (worlds[conf.level].xBlock (_pos,1)) oled.print ("OK", 127-36, 127-8, WHITE,  1);
        
        oled.print (String( worlds[conf.level].xBlock (_pos,-1)), 127-36, 127-16, WHITE,  1);
        oled.print (String( worlds[conf.level].xBlock (_pos,1)), 127-12, 127-16, WHITE,  1);

        oled.print (String( worlds[conf.level].yBlock (_pos,-1)), 127-24, 127-24, WHITE,  1);
        oled.print (String( worlds[conf.level].yBlock (_pos,1)), 127-24, 127-8, WHITE,  1);

        oled.print (String( worlds[conf.level].getDamage(_pos) ), 127-24, 127-8, WHITE,  1);

         oled.print (String(_pos.x), 127-6*9, 0, WHITE,  1);
  
    // *******************************************************************************  
*/
  if (swUp.shortPress() ) return HIGH; 
  else return LOW;
}


boolean progAbout() 
{
  //oled.fillRect(0, 0, 128, 128, BLACK); 

  //player.display();
  oled.screenShade (2);
  
  oled.printXYColorSize ("Prog. Version"  ,   0, 1*8,   WHITE,  1); 
  oled.printXYColorSize (PROG_VERSION  ,   0, 2*8+1,   WHITE,  1); 
  
  oled.printXYColorSize ("Programer :",   0, 4*8,   WHITE,  1); 
  oled.printXYColorSize ("Guillaume FOUSSARD",   0, 5*8+2,   WHITE,  1); 
  
  oled.printXYColorSize ("Game designers :",   0, 7*8,   WHITE,  1); 
  oled.printXYColorSize ("Mathieu FOUSSARD",   0, 8*8+2,   WHITE,  1); 
  oled.printXYColorSize ("Benoit CHANCEL(Deputy)",   0, 9*8+4,   WHITE,  1); 
  
  if (swUp.shortPress()  || swJoyLeft.shortPress()) return HIGH; 
  else return LOW;
}

boolean goToSleep() 
{
      oled.setToSleep();    // switch off the screen
      
      delay (100);

      SPI.end();
      Wire.end();
      
      digitalWrite (DISPLAY_CLK, LOW);
      delay (100);
      
      Snooze.hibernate( config_teensy3x );

      swUp.handle(); swMid.handle();  swDown.handle();    
      
      sleepTime = 0;
      
      mpu.begin();
      clock.begin(); // horloge
      oled.begin();

      return HIGH;
}


boolean saveEeprom() 
{
  if (swUp.shortPress()  || swJoyLeft.shortPress()) return HIGH; 
  else 
      {
        if (swDown.longPress())
            {
              EEPROM.put(0, conf);
              return HIGH;
            }
         else 
            {
              oled.printXYColorSize ("PRESS LONG TO SAVE", 10 ,  80, RED,  1);
              //oled.print (String(count), 10 ,  90, WHITE,  1);
              return LOW;
            }
      }
}

boolean resetWorlds() 
{
  conf.maxLevel = 0;
  conf.level = 0;
  
  worldsItems[0]._world->setEnable(HIGH);                                  //  initialisation de la validité des mondes  
  for (byte i=1; i < nbworlds; i++) worldsItems[i]._world->setEnable(LOW); //  initialisation de la validité des mondes  
  
  return HIGH;
}

boolean freezeScreen()
{
  swUp.handle(); swMid.handle();  swDown.handle(); 

  while (!Switch::anyShortPress()) 
    {
      swUp.handle(); swMid.handle();  swDown.handle();
      SPI.end();
      delay (100);
    }
  mpu.begin();
  clock.begin(); // horloge
  oled.begin();
  
  return HIGH; 
  
}
