#include <Arduino.h>
#include "Config.h"
#include "Input.h"
#include <FastSSD1351.h>
#include "Player.h"

extern FastSSD1351 oled;   // objet graphique
extern Switch  swUp, swMid, swDown ;                 // declaration objet switch , swJoy 
extern Switch  swJoyRight, swJoyLeft, swJoyDown, swJoyUp;            // declaration objet switch sur mouvemet joystick
extern Joystick  joyX;                 // declaration objet joystick
extern Joystick  joyY;  
extern Conf conf;
extern World *worlds;


// Methodes PUBLIQUES Player---------------------------------------------------------------------------------------------

Player::Player (Skin skin)  : _skin(skin)
      {
        _frameNb = _skin.frameNb;
        _mask =    { _skin.widthMask, _skin.height  };
        
        _overLapX = ( _skin.width - _skin.widthMask) / 2 ;
          
      }

void Player::begin(World world)  // positionne le joueur au début du monde       
     {
      worlds[conf.level].begin();
      _pos.x = world.getStart().x*8;
      _pos.y = world.getStart().y*8;
      _sight.x = worlds[conf.level].getStartDir();
      _getKey = LOW;
      _getSword = LOW;
      _life=127;
     }

Vector  Player::getPos()
  {
    Vector r;
    r.x = _pos.x;
    r.y = _pos.y;
    return r;
  }

boolean Player::getFinish(World world)
  {
    return  _pos.x == world.getFinish().x * 8 && _pos.y == world.getFinish().y*8 ;
  }

Vector  Player::getScroll()
  {
    Vector r;
    r.x = _scroll.x;
    r.y = _scroll.y;
    return r;
  }

Vector Player::getSight()
  {
    return _sight;
  }

int16_t Player::getLife()
  {
    return _life;
  }


void  Player::play(World world, Vector joy, boolean jump, boolean action)  // PLAYING IN THE WORLD
    {
          
          move(world, joy, jump, action );
          
          _setScroll();                                   // define the scroll position following player position 
                      
          world.display(_scroll);                         // display world in normal mode 
          
          oled.drawBitmap16bit (                          // display the player in the world
                                  _skin.skin,
                                  _frameNb * _skin.width,
                                  
                                  _pos.x - _overLapX - _scroll.x,       _pos.y-_scroll.y,
                                  _frame * _skin.width,    0,
                                  _skin.width,            _skin.height, 
                                  
                                  0xFFFF,
                                  _sight.x
                                  ); 
          
      // gestion de la partie basse de l'ecran
          _displayLife ( 2 );    
          if (_getKey )    world.printKey ( 127-16, 127-16 );
          if (_getSword )  world.printSword ( 127-32, 127-16 );

      
    }



void  Player::display(World world, Vector scroll)  // PLAYING IN THE WORLD
    {
          oled.drawBitmap16bit (                         // display the player in the world
                                    _skin.skin,
                                    _frameNb * _skin.width,
                                    
                                    _pos.x - _overLapX - scroll.x,       _pos.y - scroll.y,
                                    _frame * _skin.width,    0,
                                    _skin.width,            _skin.height, 
                                    
                                    0xFFFF,
                                    _sight.x
                                    ); 

    }

void  Player::move(World world, Vector joy, boolean jump, boolean action)
  {

#define GRAVITE 3
#define IMPULSION 27
#define SPEEDLIMIT 31

// direction du regard
    
    if (joy.x > 0) _sight.x = 1; if (joy.x < 0) _sight.x = -1;
    if (joy.y > 0) _sight.y = 1; if (joy.y < 0) _sight.y = -1;

// mouvement de la marche du joueur
      if (joy.x!=0)  
            {
              if (_timeWalk > 60  )  // marche 
                {
                  _timeWalk = 0;
                  _frame+=_sight.x;
                }
            }
      else 
           {
              if (_sight.x>0) _frame=0;    // position statique
                 else if (_sight.x<0) _frame=7;  
            }
           
     _frame &= 7; 

// gestion de la gravité et des sauts
    
    if ( world.yMoveCheck (_pos, 1, _mask)) 
          {
            if ( jump && !world.yMoveCheck (_pos, -1, _mask ) ) _dy=-IMPULSION; // Le player saute
            
            if ( int8_t damage = world.getDamage ( _pos, _mask )  ) // aie !
                  {  
                  _life -=damage; 
                  if (conf.enableSound) tone (5,110,20);
                  if (_life<0) _life=0;
                  }
          }
    else _dy+=GRAVITE; // le player est en chute libre soumis à la gravité
      
    if (_dy > SPEEDLIMIT) _dy = SPEEDLIMIT;  // limitation de la vitesse
    if (_dy < -SPEEDLIMIT) _dy = -SPEEDLIMIT;
    
// clash along X

    int8_t dirx = joy.x/abs(joy.x);
    
    if ( !world.xMoveCheck (_pos, joy.x, _mask) ) _pos.x+= joy.x; // on decale le joueur si il y a de la place pour faire le mouvement dx
    else 
      {
        while ( !world.xMoveCheck (_pos, dirx , _mask ) ) _pos.x+=  dirx;                         // we translate the player to the stop 

        if ( world.xMoveCheck (_pos, joy.x, _mask ) == 2 && !world.yMoveCheck (_pos, -1, _mask) )                 // if there  a step and enough place on the top => the player climb the step 
              {
                _pos.x += _sight.x;
                uint8_t h = _pos.y & 7;
                
                if (  h == 0   ) _pos.y -= 8;
                else _pos.y -= h;
              }
      }

// limit of the world

    _limitVector ( &_pos,      _overLapX,    (worlds[conf.level].getBlockMapWidth() << 3 )  - _skin.width + 1,
                               0,            (worlds[conf.level].getBlockMapHeight() << 3 )  - _skin.height +1);

// clash along Y
    int8_t diry = _dy/abs(_dy);
    
    if ( !world.yMoveCheck (_pos, _dy >> 2 , _mask ) ) _pos.y += _dy >> 2;       // ( dy >> 2) ->  scale de 4 sur dy 
    else 
      {
        while ( !world.yMoveCheck (_pos, diry , _mask) ) _pos.y += diry;
      }



// take the objects

    objectTypeType o = world.getObject ( _pos , _mask );

    if ( o == LIFE ) 
        {
          _life +=16; 
          if (_life > 127) _life = 127;
        }

    else if ( o == KEY ) _getKey = HIGH; 
         else if ( o == SWORD ) _getSword = HIGH; 

    
// check if there is a door to open
   
   int8_t d = world.xMoveDoorCheck (_pos, _sight.x, _mask );

   
   if (d != -1 && _getKey )
      {
         if ( action )
              {
                 world.setDoorState (d , OPEN);
                 _getKey = LOW;
              }
      }
   }

  
// Methodes  PRIVATE -----------------------------------------------------------------------------------------------------------------

void Player::_setScroll()  // positionnemet de l'ecran par rapport au joueur
  {
    if (_sight.x==1) _offset.x+=1; else _offset.x-=1;

    if (_sight.y==1) _offset.y+=1; else _offset.y-=1;

    _limitVector ( &_offset, -25,  25, -25,  25);
    
    _scroll.x = _pos.x - (63-12) + _offset.x;
    _scroll.y = _pos.y - (63-16) + _offset.y;

    _limitVector ( &_scroll,  0,  (worlds[conf.level].getBlockMapWidth() << 3 )  - 129,
                              0,  (worlds[conf.level].getBlockMapHeight() << 3 )  - 129);
  }

void Player::_displayLife(uint8_t x)   // affiche la vie en bas de l'écran
      {
        uint8_t c = (_life + 1) >> 4 ;
        uint8_t r = (_life + 1) & 0b1111 ;
        
        for (uint8_t i=0; i<c; i++)
         
         oled.drawBitmap16bit (         coeur,                        //  coeurs entiers 
                                        10,                           
                                        x+12*i, 116,                 
                                        0, 0,                         
                                        10, 10,                       
                                        0xFFFF,
                                        1
                                        );  
         
        oled.drawBitmap16bit (          coeur,                        // coeur partiels
                                        10,                           
                                        x+12*c, 116,                 
                                        0, 0,                         
                                        r, 10,                        
                                        0xFFFF,
                                        1
                                        ); 
        }

void  Player::_limitVector (Vector *v, int16_t xmin, int16_t xmax, int16_t ymin, int16_t ymax) // définit le vacteur d'entrer entre minimax
    {
    if (v->x>xmax)  v->x=xmax;  if (v->x<xmin)  v->x=xmin;
    
    if (v->y>ymax)  v->y=ymax;  if (v->y<ymin)  v->y=ymin;
    }

void  Player::_limit (int16_t *x, int16_t xmin, int16_t xmax)
    {
    if (*x>xmax)  *x=xmax;  if (*x<xmin)  *x=xmin;
    }
