#include <Arduino.h>
#include "Config.h"
#include "TimeClock.h"
#include "Input.h"
#include <FastSSD1351.h>
#include "World.h"

extern Switch  swUp, swMid, swDown ;                 // declaration objet switch , swJoy
extern Switch  swJoyRight, swJoyLeft, swJoyDown, swJoyUp;            // declaration objet switch sur mouvemet joystick
extern FastSSD1351 oled;
extern Conf conf;
extern TimeClock clock;
  
TimeClock::TimeClock( ) { }

boolean TimeClock::display()
          {
         clock.draw(WHITE, BLACK) ;

         if (Switch::anyShortPress())  return HIGH; 
         else return LOW;
          }

void TimeClock::begin()
        {
            setSyncProvider(TimeClock::_getTeensy3Time);
            adjustTime(2*3600);  

            /*
            Serial.begin(115200);
            
            //while (!Serial);  // Wait for Arduino Serial Monitor to open
            delay(100);
            
            if (timeStatus()!= timeSet) 
            {
            Serial.println("Unable to sync with the RTC");
            } 
            else 
            {
            Serial.println("RTC has set the system time");
            }*/
        }


String TimeClock::getTimeString ()
        {
         String s;
         s =  _digitsToString( hour());
         s+= ":";
         s+= _digitsToString( minute());
         s+= ":";
         s+= _digitsToString( second());
         return s;
        }

String TimeClock::getDateString ()
        {
         String s;
         s =  _digitsToString( day());
         s+= "/";
         s+= _digitsToString( month());
         s+= "/";
         s+=  String(year());
         return s;
        }

void  TimeClock::handle()
{
  //Serial.println("hello World");
  if (Serial.available()) 
          {
          time_t t = _processSyncMessage() ;
          if (t != 0) 
              {
                Teensy3Clock.set(t); // set the RTC
                setTime(t);
              }
          }
  
}

void TimeClock::draw(uint16_t digitsColor, uint16_t backGroundColor)     // AFFICHAGE DE L'HEURE EN 7 SEGMENTS
     {
        oled.fillRect(0, 0, S_WIDTH, S_HEIGHT, backGroundColor);

        byte h=hour();
        byte hd=hour()/10;
        byte hu=h-10*hd;
        
        oled.drawDigit7seg ( hd,
                             0, 38,  // position d'affichage sur l'écran
                             0x0001,           // couleur fond 0 (transparent si 0x0001)
                             digitsColor  );        // couleur des segments 1
        oled.drawDigit7seg ( hu,
                             30, 38,  // position d'affichage sur l'écran
                             0x0001,           // couleur fond 0 (transparent si 0x0001)
                             digitsColor  );        // couleur des segments 1
        
        byte m=minute();
        byte md=m/10;
        byte mu=m-10*md;
        
        oled.drawDigit7seg ( md,
                             69, 38,  // position d'affichage sur l'écran
                             0x0001,           // couleur fond 0 (transparent si 0x0001)
                             digitsColor  );        // couleur des segments 1
        
        oled.drawDigit7seg ( mu,
                             99, 38,  // position d'affichage sur l'écran
                             0x0001,           // couleur fond 0 (transparent si 0x0001)
                             digitsColor  );        // couleur des segments 1
        
        if (second() & 1) {
          oled.fillRect(62, 63-4-5, 5, 5, digitsColor);
          oled.fillRect(62, 63+4+5, 5, 5, digitsColor); 
        }
        
        oled.printXYColorSize(getDateString () , 0,0, digitsColor, 1);     
 }

String TimeClock::_digitsToString (uint8_t digits) 
        {
         String s;
         if (digits<10) s="0"+String(digits);
         else s=String(digits);
         return s;
         }


time_t TimeClock::_getTeensy3Time()
      {
        return Teensy3Clock.get() + 2*3600;
      }

#define TIME_HEADER  "T"   // Header tag for serial time sync message
    
unsigned long TimeClock::_processSyncMessage() 
    {
      
      unsigned long pctime = 0L;
      const unsigned long DEFAULT_TIME = 1357041600; // Jan 1 2013 
    
      if(Serial.find(TIME_HEADER)) 
      {
         pctime = Serial.parseInt() ;
         
         return pctime;
         if( pctime < DEFAULT_TIME) { // check the value is a valid time (greater than Jan 1 2013)
           pctime = 0L; // return 0 to indicate that the time is not valid
         }
      }
      return pctime;
    }
