
#include <Arduino.h>
#include "Config.h"
#include <FastSSD1351.h>
#include "World.h"
#include "WData.h"

extern Conf conf;
extern FastSSD1351 oled;
extern Gyro mpu;

#define X_MASK_RIGHT  19   // over 23 to take accound of the shape of the skin
#define X_MASK_LEFT   4    // over 0  to take accound of the shape of the skin

// creating worlds
// WORLD 0 ---------------------------------------------------------------------------------------------------------
    // Packdegats 
                             
    const int8_t startDir0 = -1;                         
    const int8_t damage0[16]  {  
                                0,  0,  0,  0,
                                0,  0,  0,  0,
                                1,  0,  0,  0,
                                0,  0,  0,  0 
                                }; 

    const objectTypeType objectType0[16] {  
                                BACKGROUND,   BACKGROUND,  BACKGROUND,  BACKGROUND,
                                SWORD,        LIFE,        BACKGROUND,  BACKGROUND,
                                BACKGROUND,   BACKGROUND,  BACKGROUND,  BACKGROUND,
                                KEY,          BACKGROUND,  BACKGROUND,  BACKGROUND
                                }; 
               
    const World world0  ("The magic forest" ,  //   name  blockmap 
                          
                          blockMap0, 
                          blockMapWidth0 , 
                          blockMapHeight0 , 
                          
                          packTexture0, 

                          doors0,
                          doorBlocksNb0,
                          
                          objectsList0, 
                          objectsNb0, 
                          objects0,
                          objectType0, 
                          
                          backGround0, 
                          backGroundWidth0, 
                          
                          startDir0,
                          start0,  finish0,       
                          damage0                               
                          
                          ); 

// WORLD 1 ---------------------------------------------------------------------------------------------------------
  
    const int8_t startDir1 = -1;
    
    // Packdegats
                               // 0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
    const int8_t   damage1[16] {  
                                  0,  2,  2,  0,
                                  1,  0,  0,  0,
                                  0,  0,  0,  0,
                                  1,  0,  0,  0 
                                  }; 

    const objectTypeType objectType1[16] {  
                                BACKGROUND,   BACKGROUND,   BACKGROUND,   BACKGROUND,
                                BACKGROUND,   BACKGROUND,   BACKGROUND,   BACKGROUND,
                                LIFE,         BACKGROUND,   BACKGROUND,   BACKGROUND,
                                KEY,          BACKGROUND,   BACKGROUND,   SWORD 
                                };

    
    const World world1  ("The icy montains" ,
                          
                          blockMap1, 
                          blockMapWidth1, 
                          blockMapHeight1,  
                          
                          packTexture1,

                          doors1,
                          doorBlocksNb1,         
                            
                          objectsList1, 
                          objectsNb1, 
                          objects1,
                          objectType1, 
                          
                          backGround1, 
                          backGroundWidth1, 
                          
                          startDir1,
                          start1,  finish1,       
                          damage1 
                          ); 
  
// WORLD 2 ---------------------------------------------------------------------------------------------------------
    
    const int8_t startDir2 = 1;
    // Packdegats
    const int8_t damage2[16] {  
                                0,  1,  0,  0,
                                2,  0,  0,  0,
                                0,  0,  0,  0,
                                0,  0,  0,  0 
                                }; 
    
    const objectTypeType objectType2[16] {  
                                BACKGROUND,   BACKGROUND,   BACKGROUND,   BACKGROUND,
                                BACKGROUND,   BACKGROUND,   BACKGROUND,   BACKGROUND,
                                LIFE,         BACKGROUND,   BACKGROUND,   BACKGROUND,
                                KEY,          BACKGROUND,   BACKGROUND,   SWORD         
                                };
                                
    const World world2  ("The great reef" , 
                          
                          blockMap2, 
                          blockMapWidth2, 
                          blockMapHeight2, 
                          
                          packTexture2, 

                          doors2,
                          doorBlocksNb2,
                          
                          objectsList2, 
                          objectsNb2, 
                          objects2, 
                          objectType2, 
                          
                          backGround2, 
                          backGroundWidth2,  
                          
                          startDir2,
                          start2,  finish2,       
                          
                          damage2 
                          );

  
// WORLD 3 ---------------------------------------------------------------------------------------------------------
   
    const int8_t startDir3 = 1;
    const int8_t   damage3[16] {  
                                0,  2,  2,  0,
                                3,  0,  0,  0,
                                0,  0,  1,  1,
                                1,  0,  0,  0 
                                }; 
    
    const objectTypeType objectType3[16] {  
                                BACKGROUND,   BACKGROUND,   BACKGROUND,   BACKGROUND,
                                BACKGROUND,   BACKGROUND,   BACKGROUND,   BACKGROUND,
                                LIFE,         BACKGROUND,   BACKGROUND,   BACKGROUND,
                                KEY,          BACKGROUND,   BACKGROUND,   SWORD 
                                };
    
    const World world3  ("The nether" ,
                          blockMap3, 
                          blockMapWidth3, 
                          blockMapHeight3, 
                          
                          packTexture3,  

                          doors3,
                          doorBlocksNb3,
                          
                          objectsList3, 
                          objectsNb3, 
                          objects3,
                          objectType3, 
                          
                          backGround3, 
                          backGroundWidth3,   
                          
                          startDir3,
                          start3,  finish3,       
                          damage3 
                          ); 

// WORLDS ---------------------------------------------------------------------------------------------------------

    const World * worlds[nbworlds] = {  &world0,   &world1,    &world2,    &world3  };

// Constructor of worlds objects  ------------------------------------------------------------------------------

World::World  (  
            String          worldName,
            
            const uint16_t *blockMap,
            const uint8_t   blockMapWidth,       // nb of blocks  
            const uint8_t   blockMapHeight,      // nb of blocks  
            
            const uint16_t *packTexture,

            const uint8_t   *doors,               // list of blocks of doors
            const uint8_t   doorBlocksNb,         // nb of block doors      
            
            const uint8_t  *objectsList, 
            const uint16_t  objectsNb,
            const uint16_t *objects,
            const objectTypeType *objectType,    // object type
            
            const uint16_t *backGround,
            const uint8_t   backGroundWidth,
            
            const int8_t   startDir,
            Vector          start,
            Vector          finish,
            const int8_t    *damage
                                  ) : _name(worldName),
                                  _blockMap(blockMap),
                                  _blockMapWidth (blockMapWidth),
                                  _blockMapHeight (blockMapHeight),
                                  
                                  _packTexture(packTexture),

                                  _doors(doors),
                                  _doorBlocksNb(doorBlocksNb),

                                  _objectsList(objectsList),
                                  _objectsNb(objectsNb),
                                  _objects(objects),
                                  _objectType(objectType),
                                  
                                  _backGround(backGround),
                                  _backGroundWidth(backGroundWidth),

                                  _startDir(startDir),
                                  _start(start),  _finish(finish),
                                  
                                  _damage(damage)
      {   
       // for map overview mode  ------------------------------------------------------
           
           for (uint8_t i=0; i < 16; i++)  _averageColorPackTexture[ i ] = _getAverageColorBlock ( i );  // save the average values of each blocks of packtexture
      
       // Initialize the doors ------------------------------------------------------
       
           _doorsNb = _doors[(_doorBlocksNb - 1) << 2 ] + 1;
           _doorState = (doorStateType *) malloc(sizeof(doorStateType) * _doorBlocksNb); 
          
      // Initialize the objects ------------------------------------------------------
            
            _objectState = (objectStateType *) malloc(sizeof(objectStateType) * _objectsNb); // allocate mem for the array of objects 
             
            for (uint16_t i=0; i < 16; i++) 
                  {
                    if (_objectType[ i ] == KEY)    _keyType = i;
                    if (_objectType[ i ] == SWORD)  _swordType = i;
                  }
                   
            begin();   
      }




// ---------------------------------------------------------------------------------------------------------------------------------------------------------
void World::begin()
  {
    for (uint8_t i=0; i < _doorsNb; i++) setDoorState ( i, CLOSED) ;
    
    for (uint16_t i=0; i < _objectsNb; i++) _objectState[i] = DROP; 
  }

void World::move ()
  {
  
  }

// ---------------------------------------------------------------------------------------------------------------------------------------------------------
objectTypeType World::getObject (Vector pos, Vector mask )
  
  {
      for ( uint16_t i=0; i<_objectsNb; i++)   // counting over the list of objects
        {
           if ( _objectState[i] == DROP)
            {
              uint16_t xo = _objectsList [ i * 3 + 1 ] << 3 ;  // x absolute position of the object in the map
              uint16_t yo = _objectsList [ i * 3 + 2 ] << 3 ;  // y absolute position of the object in the map
      
              if ( 
                      (pos.x + mask.x) > xo && pos.x  < (xo + 16) && 
                      (pos.y + mask.y) > yo && pos.y < (yo + 16)
                      ) // check if object is on pos
              
                 {
                    if (_objectType [_objectsList[ 3 * i] ] != BACKGROUND ) 
                      {
                        _objectState[i] = TAKEN;
                        return _objectType [_objectsList[ 3 * i] ];  // object type;
                      }
                 }
             } 
         } 
      return BACKGROUND;
  }




// ---------------------------------------------------------------------------------------------------------------------------------------------------------
void World::printKey ( uint8_t x, uint8_t y )
  {       
      _printObject ( _keyType, x, y );
  }

void World::printSword ( uint8_t x, uint8_t y )
  {       
      _printObject ( _swordType, x, y );
  }


// ---------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t World::getBlockType (uint16_t x, uint16_t y )
  {     
  return 
                                    _blockMap[ (( y >> 3 ) * (_blockMapWidth >> 2 )  ) + ( x >> 5 )  ]    // get the uint16 in blockMap array
                                     >>  
                                     ( ( 3-( (x >> 3) & 0b11)  ) << 2)                                    // get the hex digit in the uint16  
                                     
                                     & 0xF;                                                               // keep the last hex digit
   }

uint8_t World::getBlockPass (uint16_t x, uint16_t y )
  {     
    // check if the position is in the the door array 
        for (uint8_t i=0; i < _doorBlocksNb; i++) 
        {
          if ( 
                  _doors[(i << 2) | 2 ] == ( x >> 3 ) && 
                  _doors[(i << 2) | 3 ] == ( y >> 3 ) 
              ) 
              {
                if ( _doorState [ i ] == CLOSED) return _doors[(i << 2) | 1 ];
                else return 0xF;
              } 
        }
     // if the position is not in the door array return the value of block from blockMap array   
        
        return getBlockType ( x, y );                                                
   }


// ---------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t World::getDamage (Vector pos , Vector mask )
    {
   // gives the checking zone below pos 
        pos.y += mask.y ;  
        
   // downside row checking 
        uint8_t d = 0;
        uint8_t widthScan = mask.x >> 3;
        
        if ( (pos.x & 7) == 0) for (uint8_t i=0; i < widthScan; i++) d +=  _damage[ getBlockType (pos.x + (i << 3), pos.y) ];   // if the player is just aligned on blocks grid
   
        else    //  if the player is crossing blocks grid
          {
            for (uint8_t i=0; i < widthScan + 1 ; i++)  d +=  _damage[ getBlockType (pos.x + (i << 3), pos.y) ]; //  if the player is on 4 blocks

            d = ( d * widthScan ) / ( widthScan - 1);
          }
        return d;
    }



  
// ---------------------------------------------------------------------------------------------------------------------------------------------------------
void World::display(Vector scr)  // affichage du fond et position du scroll de le la map 
{
  // print the blocksMap ----------------------------------------------------------------------------
  if (conf.enableGyro) _gy = - mpu.getPitch();       // background move with gyro
  else _gy =0;
  
  for(uint8_t j=0; j<S_HEIGHT; j++)                    // y pixel counting over the 128 pixel of the screen
      {
      uint16_t y= scr.y+j;                             // absolute position of the pix in the  map.                    
      uint8_t  n= y & 0b111;                           // position of the pix in the block 
     
          for(uint8_t i=0; i<S_WIDTH; i++)             // x pixel counting over the 128 pixel of the screen 
              {
                  uint16_t x= scr.x+i ;                 // absolute position of the pix in the  map.  
                  uint8_t  l= x & 0b111;                // position of the pix in the block 

                  uint8_t typeBlock = getBlockType   ( x, y ); 
                
                  uint8_t a = (typeBlock & 0b0011); uint8_t b = (typeBlock & 0b1100) >> 2; // relative position of the 8x8 image block in the packtextureSet of 32x32 
                 
                  uint16_t pix= _packTexture[(((b << 3) | n) << 5) | (a << 3) | l ];  // we extract the  pix in the texturePack  (32x32) at position X=8*a+l Y=8*b+n
                
                  if (pix!=0XFFFF) oled.drawPixel (i, j, pix); // print blocks from map
                
                  else  // print back ground 
                          {
                            uint16_t r = (x - (scr.x >> 1)  )              // coordinate of current pix (x y on screen) are translated with scroll vector in the background image
                                              & (_backGroundWidth -1 ) ;   // and the we crop because backGround image size is different of screen size 
                            
                            uint16_t s = (y - (scr.y >> 1) + _gy )
                                              & (_backGroundWidth -1 ) ;  
                           
                            oled.drawPixel (i, j, _backGround[(s * _backGroundWidth) + r ]);
                          }
             } 
       }
       
    // print the objects  -------------------------------------------------------------------------------
                
    for ( uint16_t nbO=0; nbO<_objectsNb; nbO++)   // counting per objects
      {
       if ( _objectState[nbO] == DROP)
        {
          uint16_t xo = _objectsList [ nbO * 3 + 1 ] << 3 ;  // x absolute position of the object in the map
          uint16_t yo = _objectsList [ nbO * 3 + 2 ] << 3 ;  // y absolute position of the object in the map
    
          if ( xo - scr.x > -17 && yo - scr.y > -17 && xo - scr.x < S_WIDTH && yo - scr.y < S_HEIGHT ) // check if object is on the screen
            {
                uint8_t o = _objectsList [ nbO * 3 ];  // object type
                uint8_t a = (o & 0b0011); uint8_t b = (o & 0b1100) >> 2; // relative position of the 16x16 image object in the objectSet of 64x64              
                
                for(uint8_t j=0; j<16; j++)                     // Y pixels compting over 16
                    {
                        for(uint8_t i=0; i<16; i++)             //  X pixels compting over 16
                            {
                            int16_t x = xo + i - scr.x ;            // y relative position of pix on the screen 
                            int16_t y = yo + j - scr.y ;            // x relative position of pix on the screen    
          
                              if ( x > -1 && x < S_WIDTH && y > -1 && y < S_HEIGHT )  // if on screen
                                {
                                  uint16_t pix= _objects[ (((b << 4) | j) << 6 ) | (a << 4) | i ];  // extracting pixel value in the objectSet (64x64) at the position X= 16*a + i  Y= 16*b + j
                                  
                                  if (pix!=0XFFFF) oled.drawPixel (x, y, pix); // print pix on screen
                                }
                            }
                    }
             }
          
         }
     }

      
  // print doors -----------------------------------------------------------------------------
    /*
    oled.printXYColorSize (String(_doorsSize[0])  ,   0, 0,   WHITE,  1); 
    oled.printXYColorSize (String(_doorsSize[1])  ,   0, 8,   WHITE,  1); 
    oled.printXYColorSize (String(_doorsSize[2])  ,   0, 16,   WHITE,  1); 
    oled.printXYColorSize (String(_doorsSize[3])  ,   0, 24,   WHITE,  1); 
    */
     
    for (uint8_t blockIndex=0; blockIndex < _doorBlocksNb; blockIndex++) 
      {
            uint16_t xd = _doors[(blockIndex << 2) | 2 ] << 3 ; // x absolute position of the block of the door in the map
            uint16_t yd = _doors[(blockIndex << 2) | 3 ] << 3 ;
          
            if ( xd - scr.x > -9 && yd - scr.y > -9 && xd - scr.x < S_WIDTH && yd - scr.y < S_HEIGHT ) // check if the door is on the screen
              {
                uint8_t typeBlock = _doors[(blockIndex << 2 ) | 1 ];        // block type
                uint8_t door = _doors[ blockIndex << 2 ];                   // number of the current door
                
                for(uint8_t j=0; j<8; j++)                                  // Y pixels compting over 8
                        {
                            for(uint8_t i=0; i<8; i++)                      //  X pixels compting over 8
                                {
                                int16_t x = xd + i - scr.x ;                // y relative position of pix on the screen 
                                int16_t y = yd + j - scr.y ;                // x relative position of pix on the screen    
              
                                if ( x > -1 && x < S_WIDTH && y > -1 && y < S_HEIGHT )  // if on screen
                                    {
                                        uint8_t a = (typeBlock & 0b0011); uint8_t b = (typeBlock & 0b1100) >> 2; // relative position of the 8x8 image block in the packtextureSet of 32x32              
                                        
                                        uint16_t pix= _packTexture[(((b << 3) | j) << 5) | (a << 3) | i ];       // we extract the  pix in the texturePack  (32x32) at position X=8*a+i Y=8*b+j
                                        
                                        if ( getDoorState (door) == CLOSED && pix != 0XFFFF ) oled.drawPixel (x, y, pix);          // print blocks from map
                                    }
                                }
                        }
                 }   
        }
}



// ---------------------------------------------------------------------------------------------------------------------------------------------------------
void World::displayOverview (
                           uint8_t x,                  // starting position in the map
                           Vector scroll               // square position
                            )
     {
    for(uint8_t j= 0 ; j < _blockMapHeight ; j++) // counting throught the blockmap
                  {
                       for(uint8_t i=0; i < _blockMapWidth ; i++) // 
                        {
                           uint8_t typeBlock = getBlockType   (  (i + x) << 3 , j << 3 ) ; 
                           uint16_t p = _averageColorPackTexture [ typeBlock ];
                           
                           if (p != WHITE ) oled.drawPixel (i, j, p );  
                           else  oled.drawPixel (i, j, _backGround [ ( (j & ( _backGroundWidth - 1))   * _backGroundWidth ) + (i & ( _backGroundWidth - 1) ) ]);
                        }
                  }
      // screen frame
      oled.drawRect((scroll.x >> 3) - 1 - x,   (scroll.y >> 3) - 1 ,  16 , 16 , RED);
     }


     
// ---------------------------------------------------------------------------------------------------------------------------------------------------------
int8_t    World::getStartDir() { return _startDir; }

Vector    World::getStart() { return _start; }

Vector    World::getFinish() { return _finish; }

String    World::getName() { return _name; }

boolean   World::getEnable() { return _enable; }

void      World::setEnable(boolean enable) { _enable = enable; }



// ---------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t World::xMoveCheck (Vector pos, int8_t dx, Vector mask)
  {
  // set the scanned zone
        if (dx > 0 ) 
          {
            pos.x += mask.x + dx - 1; 
          }
          
        else if (dx < 0 ) 
          {
            pos.x += dx ;
            
          }
             else return 0;                                                 // if dx == 0

//            SSSSssssssssSSSS            skin of the sprite
//    BBBBBBBBbbbbbbbbBBBBBBBBbbbbbbbb        blocks of the world
 
  // test the left or right row of blocks 
        uint8_t r=LOW;
        
        if ( (pos.y & 7) == 0)   // if scanned zoned goes over 4 blocks  
          {
            for (uint8_t i=0; i<4; i++) 
                {
                  if (
                            i==3 &&
                            r==0 &&
                            getBlockPass ( pos.x,  pos.y +  (i << 3) ) != 0xF
                            )  // for the last blocks (the bottom one) alone if higher blocks are free
                                   r = 2; 
                   
                  else r = r || ( getBlockPass ( pos.x,  pos.y +  (i << 3) ) != 0xF ) ;
                }
          }
        else   // if scanned zoned goes over 5 blocks
          {
            for (uint8_t i=0; i<5; i++) 
                {
                  if ( 
                            i==4 &&
                            r==0 && 
                            getBlockPass ( pos.x,  pos.y +  (i << 3) ) != 0xF
                            ) 
                                  r = 2 ; // for the last blocks (the bottom one) alone if higher blocks are free
                    
                  else  r = r || ( getBlockPass ( pos.x,  pos.y +  (i << 3) ) != 0xF ) ;
                }
          }
        return r;                            
}


// ---------------------------------------------------------------------------------------------------------------------------------------------------------
boolean World::yMoveCheck (Vector pos, int8_t dy, Vector mask)
   
  {
    // set the scan zone 
        
        if ( dy > 0 ) 
          {
            pos.y += mask.y - 1 + dy ; 
            if ( pos.y > (_blockMapHeight << 3 ) ) return 1;
          }
          
        else if (dy < 0) 
          {
            pos.y +=  dy ;
            if ( pos.y < 0 ) return 1;
          }
             
             else return 0;    // test si dy est null 

        
    // test la rangée de bloc au dessus ou en dessous
        boolean r = 0;
        uint8_t widthScan = mask.x >> 3;
        
        if ( (pos.x & 7) == 0) // si sa tombe juste sur 2 blocs 
          {
            for (uint8_t i=0; i < widthScan ; i++) r = r || ( getBlockPass ( pos.x +  (i << 3) , pos.y ) != 0xF ) ;
          }
        else                  // si on est a cheval sur 3  blocs 
          {
            for (uint8_t i=0; i< widthScan +1; i++) r = r || ( getBlockPass ( pos.x  +  (i << 3) , pos.y ) != 0xF ) ;
          }
        return r;                                                
  }

// ---------------------------------------------------------------------------------------------------------------------------------------------------------

int8_t  World::xMoveDoorCheck (Vector pos, int8_t sx, Vector mask )              // if there is there a closed door ? return number of the door else -1
  { 

   uint16_t x = pos.x + sx;
   if ( sx > 0 ) 
              x = ( x + mask.x ) >> 3 ; // check on right position
   
   else  
              x = x >> 3;           // check on left position
   
   uint16_t y = pos.y >> 3;

   // check if the x position is in the the door array 
        for (uint8_t i=0; i < _doorBlocksNb; i++) 
        {
           if ( _doorState [ i ] == CLOSED)
                {
                    uint8_t xd = _doors[(i << 2) | 2 ];
                    uint8_t yd = _doors[(i << 2) | 3 ];
                    
                    if ( y == yd && x == xd ) return _doors[ i << 2];
                }
         }
    return -1; 
  }


// ---------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t World::getBlockMapWidth ()  { return _blockMapWidth; }

uint8_t World::getBlockMapHeight ()  { return _blockMapHeight;  }

void World::setDoorState (uint8_t door , doorStateType state)
  {
         for (uint8_t i=0; i < _doorBlocksNb; i++) 
         {
          if (_doors[ i << 2 ] == door) _doorState [ i ] = state;
         }
  }

// ---------------------------------------------------------------------------------------------------------------------------------------------------------
doorStateType  World::getDoorState (uint8_t door)
    {
        for (uint8_t i=0; i < _doorBlocksNb; i++) if (_doors[ i << 2 ] == door) return _doorState [ i ];
        return CLOSED;  // default value
    }

    
// PRIVATE **************************************************************************************************************************

void World::_printObject ( uint8_t o, uint8_t x, uint8_t y )
  {       
      uint8_t a = (o & 0b0011); uint8_t b = (o & 0b1100) >> 2; // relative position of the 16x16 image object in the objectSet of 64x64 
                              
      for(uint8_t j=0; j<16; j++)                     // Y pixels compting over 16
          {
              for(uint8_t i=0; i<16; i++)             //  X pixels compting over 16
                  {
                   
                  uint16_t pix= _objects[ (((b << 4) | j) << 6 ) | (a << 4) | i ];  // extracting pixel value in the objectSet (64x64) at the position X= 16*a + i  Y= 16*b + j
                  
                  if (pix!=0XFFFF) oled.drawPixel (x + i , y + j , pix); // print pix on screen
                  }
          }
  }



// ---------------------------------------------------------------------------------------------------------------------------------------------------------
uint16_t World::_getAverageColorBlock ( uint8_t typeBlock   )
       { 
         uint8_t a = (typeBlock & 0b0011); uint8_t b = (typeBlock & 0b1100) >> 2;
                                                        
          uint16_t red = 0 , green= 0, blue= 0;
             
              for(uint8_t v=0; v < 8 ; v++) // compte le nombre de pixel dans un bloc DU PACK TEXTURE
                    {
                      for(uint8_t u=0; u < 8 ; u++)
                            {
                              uint16_t p= _packTexture[  
                                                         32 * ( (b << 3) + v) +
                                                                (a << 3) + u 
                                                                             ];     // on extrait le pixel dans le pack texture (32x32) à la position X=8*a+l Y=8*b+n
                                                     
                                    red +=   (p & 0b1111100000000000) >> 11; // On additionne toutes les valeur de rouge
                                    green += (p & 0b0000011111100000) >> 5;  // On additionne toutes les valeur de vert
                                    blue +=  (p & 0b0000000000011111)  ;     // On additionne toutes les valeur de bleu 
                            }
                    }

              red    = (red   >> 6 ) << 11;  // we calculate the average by dividing by  64 (>> 6) and we move to left 
              green  = (green >> 6 )  << 5;
              blue   = (blue  >> 6 ) ;
                    
              return (uint16_t)(red | green | blue);                                        
       }
