
#ifndef WORLD_H
#define WORLD_H

#include "Config.h"
#include "Input.h"

enum doorStateType      { CLOSED, OPEN };                           // state of doors 
enum objectTypeType     { BACKGROUND, KEY, LIFE, SWORD, PICKAXE};   // behaviour of the objects
enum objectStateType    { DROP, TAKEN };                            // to define if the objects are DROP in the map or TAKEN with the player

class World
{
public : 
      World(
                      String          name,
                      const uint16_t  *blockMap,       // blockmap
                      const uint8_t   blockMapWidth,   // nb de bloc en largeur 
                      const uint8_t   blockMapHeight,   // nb de bloc en largeur 
                      
                      const uint16_t  *packTexture,    // pack texture codé en blocs de 8x8 dans un tableau de 32x32 sur des uint16_t

                      const uint8_t   *doors,          // list of blocks of doors 
                      const uint8_t   doorBlocksNb,     // nb of block doors     
                      
                      const uint8_t   *objectsList,    // list of objects with their position 
                      const uint16_t   objectsNb,      // Nb of objects on the map
                      const uint16_t  *objects,        // tileSet of 16 objects
                      const objectTypeType *objectType,// object type
                      
                      const uint16_t  *backGround,     // image 16 bit 
                      const uint8_t   backGroundWidth, // largeur image
                      
                      const int8_t    startDir, 
                      
                      Vector          start,           // point d'apparition
                      Vector          finish,
                     
                      const int8_t    *damage          // tableau de 16 int indiquant le dégat causé par le bloc en attibut  > 0 pour un dégat < 0 de la vie en plus !
                                   );
        
      void begin();
      
      void moveClash (Vector *pos, int8_t dx, int8_t *dy );  // moving of the player in the world with clash management
      
      void move ();                                     // internal movement of the world (doors, plateformes etc...)

      objectTypeType getObject (Vector pos, Vector mask );       // return the object type of the position KEY, SWORD, PICKAXE or BACKGROUND

      void printKey (  uint8_t x, uint8_t y );      // print the key image  objects
      void printSword (  uint8_t x, uint8_t y );    // print the sword image  objects 
                            
      uint8_t getBlockType ( uint16_t x, uint16_t y ); // return the block type of the position
      uint8_t getBlockPass ( uint16_t x, uint16_t y ); // return the block type of the position including door if any
      
      uint8_t getDamage (Vector pos, Vector mask );                  // get the damage value of the bloc at the position
      
      void displayOverview (
                          uint8_t x,                  // x position in the map
                          Vector scroll               // positon of the screen
                          );

      void display    (Vector pos);  // affiche le monde avec l'origine de l'écran sur pos
      
      int8_t    getStartDir() ;
      Vector    getStart();             // get start position of the world
      Vector    getFinish ();           // get finish position of the world
      String    getName();              // get name of the world
      
      boolean   getEnable();               // to say if the world is enable or disable
      void      setEnable(boolean enable); // to set enable the world

      uint8_t   xMoveCheck (Vector pos, int8_t dx, Vector mask);  // aucun bloc à la distance dx => 0 
                                                 // présence de bloc qui empeche de passer le joueur => 1
                                                 // présence de 1 bloc en bas et 3 bloc libres au dessus permettant le passage du joueur à la distance dx => 2 
  
      boolean   yMoveCheck (Vector pos, int8_t dy, Vector mask);  // presence de au moins un block en haut ou en bas à la distance dy

      int8_t    xMoveDoorCheck   (Vector pos, int8_t sx, Vector mask );         // if there is there a closed door ? return number of the door else -1
      
      uint8_t getBlockMapWidth ();   //                                
      uint8_t getBlockMapHeight (); 

      void           setDoorState (uint8_t door , doorStateType state); // set the door CLOSED or OPEN
      doorStateType  getDoorState (uint8_t door);                       // get the state of the door 

      

private : 

       void _printObject ( uint8_t o, uint8_t x, uint8_t y );    // print the object image corresponding to "o" type in the objects image set
          
       
       uint16_t _getAverageColorBlock ( uint8_t typeBlock  );     // calcul la couleur moyenne d'un bloc du pack texture 
  
       String   _name;
       
       const uint16_t *_blockMap ;         // tableau défini sur la map
       const uint8_t  _blockMapWidth;      // nb of blocks  
       const uint8_t  _blockMapHeight;     // nb of blocks 
        
       const uint16_t *_packTexture ;      // tableau de 16 par 16. Définition du pack de texture

       
       const uint8_t    *_doors ;           // list of blocks of the doors 
       const uint8_t    _doorBlocksNb ;     // number of total blocks of the doors
       
       uint8_t          _doorsNb;           // number of doors
       doorStateType    *_doorState;        // state of the blocks of doors open or close
       
       uint16_t _averageColorPackTexture [16];            // pour l'enregistrement des couleurs moyenne des 16 blocs du pack texture 
       
       
       boolean _enable;     // is the world enable
       
       uint8_t _gy;         // moving of the background

       const uint8_t          *_objectsList ;         // object list with their position in the map
       const uint16_t         _objectsNb ;            // nb of object in the map
       const uint16_t         *_objects ;             // tileSet of 16 object in the map
       const objectTypeType   *_objectType;           // type of objects 
       objectStateType        *_objectState ;         // using of objects
       
       uint8_t                _keyType,_swordType, pickAxeType ;    // type of object in the  4x4 set of objects (64x64 pixels)
       
       const uint16_t *_backGround ;      // backGround image
       const uint8_t   _backGroundWidth;  // width of backGround Image

       const int8_t   _startDir;
       
       Vector         _start,_finish; // starting and finishing points of the world

       const int8_t   *_damage  ;     // array of blocks that hurt
};

// hart for the life value

const uint16_t coeur[10*10] PROGMEM = {

  0XFFFF, 0XFFFF, 0XE801, 0XE800, 0XFFFF, 0XFFFF, 0XFB2B, 0XFC11, 0XFFFF, 0XFFFF,
  0XFFFF, 0XD801, 0XE801, 0XE800, 0XE800, 0XFA08, 0XF864, 0XFA08, 0XFB2C, 0XFFFF,
  0XC801, 0XD801, 0XE800, 0XF000, 0XF904, 0XF920, 0XF9A6, 0XFFDF, 0XFA08, 0XFB2C,
  0XC801, 0XD801, 0XE800, 0XF801, 0XF801, 0XF920, 0XF820, 0XF801, 0XFFDF, 0XFB2C,
  0XD801, 0XD801, 0XE800, 0XF801, 0XF920, 0XFA08, 0XFA08, 0XF9A6, 0XFFDF, 0XFB2C,
  0XC801, 0XD801, 0XE800, 0XF801, 0XF820, 0XF820, 0XF8E3, 0XF801, 0XF801, 0XFAED,
  0XFFFF, 0XD801, 0XD801, 0XF820, 0XF820, 0XF820, 0XFA08, 0XF9E7, 0XFB2C, 0XFFFF,
  0XFFFF, 0XFFFF, 0XD801, 0XE800, 0XF801, 0XF801, 0XF9E7, 0XFB2C, 0XFFFF, 0XFFFF,
  0XFFFF, 0XFFFF, 0XFFFF, 0XF000, 0XF9E7, 0XF820, 0XFB2C, 0XFFFF, 0XFFFF, 0XFFFF,
  0XFFFF, 0XFFFF, 0XFFFF, 0XFFFF, 0XF9E7, 0XFB2C, 0XFFFF, 0XFFFF, 0XFFFF, 0XFFFF };

#endif







  
